module Polynomial (
                 Polynomial, Monomial,
                 xpower, x, coefficient, power,
                 degree, constPoly,
                 quotient,
                 add, sub, neg,
                 showPoly
                  ) where


--

type Monomial   = (Integer,Integer)

-- | use this to construct a monomial. for example 5 X^2 is 
-- 5 `xpower` 2

xpower      :: Integer -> Integer -> Monomial
xpower c  p  = (c,p)

x :: Monomial 
x = 1 `xpower` 1

-- | returns the coefficent of the monomial

coefficient :: Monomial -> Integer
coefficient = fst

-- | returns the power of the variable x

power  :: Monomial -> Integer
power = snd

-- | show the monomial in TeX notation
showM :: Monomial -> String
showM m  = cstr ++ xstr ++ pstr
    where c = coefficient m
          p = power m
          cstr = if p == 0 then show c
                 else case c of
                        1  -> ""
                        -1 -> "-"
                        _  -> show c
          xstr = if p == 0 then ""
                 else "X"
          pstr = if p < 2 then ""
                 else "^{" ++ show p ++ "}"

                           

-- | multiplies two monomials

timesM  :: Monomial -> Monomial -> Monomial
timesM m n = c `xpower` r
    where c = coefficient m * coefficient n
          r = power m + power n

-- | negates a monomial

negM   :: Monomial -> Monomial
negM m = (- coefficient m) `xpower` power m

-- | quotient of monomial m divided by n (which is assumed to be monic)

quotientM m n | power m < power n = 0 `xpower` 0
              | otherwise         = coefficient m `xpower` (power m - power n)


-- Polynomials is a list of monomials in the decreasing order of their degrees
--
type Polynomial = [Monomial] 

-- | degree of the polynomial. degree of zero polynomial is -infinity here
-- it is error.

degree :: Polynomial -> Integer
degree  = power . head 

constPoly :: Integer -> Polynomial
constPoly c = [c `xpower` 0]

-- | negate the polynomail.

neg  :: Polynomial -> Polynomial 
neg  = map negM

-- | multiplies a monomial to a polynomial

times :: Monomial -> Polynomial -> Polynomial
times m = map (timesM m)

-- | adds two polynomials
add [ ]     p   =  p
add  p     [ ]  =  p
add  p      q   | degree p > degree q  = head p : add (tail p)   q
                | degree p < degree q  = head q : add    p     (tail q)
                | otherwise            = pp
                where  c  = coefficient (head p) + coefficient (head q)
                       r  = degree p
                       m  = c `xpower` r
                       ms = add (tail p) (tail q)
                       pp = if c == 0 then ms
                            else m : ms

                     
-- | subtracts two polynomials

sub    :: Polynomial -> Polynomial -> Polynomial
sub p q = add p (neg q)


-- | quotient p q divides p by q and returns the reminder. Assumes that q is monic

quotient   :: Polynomial -> Polynomial -> Polynomial
quotient []  _                       =  []
quotient p   q | degree p < degree q =  []
               | otherwise           =  pp
               where m = head p
                     n = head q
                     mp = m `quotientM` n
                     rem = p `sub` (mp `times` q)
                     pp  = mp : (rem `quotient` q)

                     







-- | shows a polynomial in TeX notation

showPoly :: Polynomial -> String
showPoly []       = ""
showPoly [m]      = showM m
showPoly (m:n:ns) = if coefficient n < 0 then
                        showM m ++ " " ++ showPoly (n:ns)
                    else
                        showM m ++ " + " ++ showPoly (n:ns)


